%clustSim(clusters)
%
%Pipeline to analize the distribution of the parameter values of a model in 
%two different groups.
%
%Add the path of the folder "clusterSimulation" of this code into the folder 
%that contains the model (model) using "addpath". The folder of the model 
%should ONLY contain a single .m file with the code that groups the solutions 
%and the .mat file with the values of the parameters and the ranges.
%
%The .m file with the model should take as an input a column vector of
%the parameters values to run the simulation and as an output 0 or 1
%depending on the group that this specific solution belongs. You can use
%the file infToy.m as a template for your model.
%
%The .mat file should contain one or two variables (use the file Solutions.mat 
%as template). One cell vector containing the names of the parameters
%and a vector/matrix satifying one of the following situations:
%   -A column vector with the values of the nominal solution around what
%    the set of solutions are search. In this situation the algorithm will
%    search in a range between 10 times lower and 10 times higher than the
%    value of the nominal solution for each parameter.
%   -A matrix with two columns. The first colums should contain the nominal
%    values for the parameters. The second column define the maximal values
%    for the parameters. The minimal value is defined as the inverse of the
%    ratio between the maximal value and the nominal value.
%   -A matrix with three columns. The first colums should contain the nominal
%    values for the parameters. The second column define the minimal values
%    for the parameters. The third column defines the maximal value for the
%    parameters.
%
%The algorith provide a figure with the distribution of the values of the
%parameters in the two different groups. Also the solutions are saved in a
%file inside the folder solutions.
%
%If it is provided the input "clusters" the algorithm displays the cluster
%analysis. The input variable should be a vector containing the indexes of
%the parameters that the user wants to select for the clusters analysis.
%
%The folder "model_test" contains an example of model to try the pipeline.
%Go to the folder and run addpath .../ClusterPipeline/clusterSimulation.
%Being ... the directory in which the folder "ClusterPipeline" is. The run 
%clustSim.
%
%Guido Santos Rosales - Version 1.3 April 13rd 2018

function clustSim(clusters)
close all
if exist('clusters','var') == 0
    plotclusters = 0;
    clusters     = [];
else
    plotclusters = 1;
end
if exist('solutions','dir') == 0
    mkdir('solutions')
end
cd solutions
save('temp.mat')
cd ..
clear all
d = dir('*.mat');
if length(d) > 1
    error('It should not be more than one .mat files in the directory')
elseif isempty(d)
    error('We can not find the .mat file with the solution in this directory!')
end
load(d.name)
clear d ans
w = who;
if length(w) > 2
    error('The .mat file should only contain two variables: one for the names and other for the values of the parameters')
end
if length(w) == 1
    namesVar = {};
end
var1 = eval(w{1});
var2 = eval(w{2});
if (iscell(var1) == 0 && iscell(var2) == 0)
    error('There is not any cell variable with the names of the parameters')
end
if (ismatrix(var1) == 0 && ismatrix(var2) == 0)
    error('There is not any vector or matrix variable with the values of the parameters')
end
if iscell(var1) == 1
    part = var2;
    nam  = var1;
else
    part = var1;
    nam  = var2;
end
if isempty(nam)
    nam = 1:length(part);
    nam = arrayfun(@(x) sprintf('%d',x),nam,'uni',false).';
end
cd solutions
load('temp.mat')
cd ..
s = size(part);
if s(2) > 3
    error('The solution file cannot have more than 3 columns')
elseif s(2) == 3
    rangos = 3;
    if sum(part(:,2) > part(:,3)) > 0
        error('There are higher values in the second column than in the third column')
    end
    if sum(part(:,2) > part(:,1)) > 0
        error('There are higher values in the second column than in the first column')
    end
    if sum(part(:,3) < part(:,1)) > 0
        error('There are lower values in the third column than in the first column')
    end
elseif s(2) == 2
    rangos = 2;
    if sum(part(:,2) < part(:,1)) > 0
        error('There are lower values in the second column than in the first column')
    end
elseif s(2) == 1
    rangos = 1;
else
    error('The solution file is empty')
end
par0    = part(:,1);
PAR     = [];
numsols = 10*length(par0);
for n = 1:numsols
    if n/10 == fix(n/10)
        PAR0  = PAR(1:end - 1,PAR(end,:) == 0);
        PAR1  = PAR(1:end - 1,PAR(end,:) == 1);
        fprintf('Iter: %g/%g\n',n,numsols)
        cd solutions
        save('Solutions.mat','PAR0','PAR1')
        cd ..
    end
    if rangos == 1
        chan = (10.^(rand(numel(par0),1)*2 - 1));
    elseif rangos == 2
        parR = part(:,2)./par0;
        chan = 10.^(rand(numel(par0),1).*(log10(parR) - log10(1./parR)) + log10(1./parR));
    elseif rangos == 3
        parR1 = part(:,3)./par0;
        parR2 = part(:,2)./par0;
        chan = 10.^(rand(numel(par0),1).*(log10(parR1) - log10(parR2)) + log10(parR2));
    end
    par  = par0.*chan;
    d = dir('*.m');
    for i = 1:length(d)
        nd(i) = strcmp(d(i).name,'clustSim.m');
    end
    warning off
    filename = d(nd==0).name;
    filename = strrep(filename,'.m',[]);
    filename = strcat('@',filename);
    out  = feval(eval(filename),par);
    chan = [chan;out];
    PAR  = [PAR,chan];
    if exist('out','var') == 0
        error('Your model does not produce an output with the index of the group for each solution')
    end
end
PAR0                      = PAR(1:end - 1,PAR(end,:) == 0);
PAR1                      = PAR(1:end - 1,PAR(end,:) == 1);
P0                        = PAR0;
P1                        = PAR1;
P0(length(P0(:,1)) + 1,:) = 0;
P1(length(P1(:,1)) + 1,:) = 1;
PARS                      = [P0,P1];
PARS                      = PARS';
cd solutions
save('Solutions.mat','PAR0','PAR1')
csvwrite('PARS.csv',PARS)
names          = nam';
names(end + 1) = {'group'};
fid            = fopen('names.csv','w');
fprintf(fid,'%s,',names{1,1:end - 1});
fprintf(fid,'%s',names{1,end});
fclose(fid);
cd ..
zip('solutions.zip','solutions');
figure(1234)
subplot(1,2,1)
boxplot(log10(PAR0'),nam)
set(gca,'XTickLabelRotation',90)
ylabel('Value of the factor of change (log10 scale)')
title('Solutions grouped in 0')
subplot(1,2,2)
boxplot(log10(PAR1'),nam)
set(gca,'XTickLabelRotation',90)
ylabel('Value of the factor of change (log10 scale)')
title('Solutions grouped in 1')
if plotclusters == 1
    clu0 = clustergram(log10(PAR0(clusters,:)),'RowLabels',nam(clusters),'Colormap',redbluecmap);
    addTitle(clu0,'Clusters group 0')
    clu1 = clustergram(log10(PAR1(clusters,:)),'RowLabels',nam(clusters),'Colormap',redbluecmap);
    addTitle(clu1,'Clusters group 1')
end
end