%infToy(par)
%
%Toy model for the simulation of the effect of some mutations on the
%inflammatory response of a cell.
%
%Simulation of a cell responding to a inflammatory signal IL-1B producing IL-8.
%
%Original publication: doi:  10.1038/s41598-017-12154-4
%Guido Santos, 13th Febrary 2018. Ver: 1.1
%

function out=infToy(par)
%Name of the variables
% variables = {'IRAK1','IRAK1p','IKK','IKKa','NFkB','mIkBa','IkBa','mIL6','IL-6','MAPK','MAPKa','AP1','AP1a','mIL10','IL-10'};
[~,y1] = ode45(@eqs_inf,[0 50],[1 0 1 0 0 0 1 0 0],[],par);
if y1(end,9) > 1
    out = 1;
else
    out = 0;
end
end

function dx = eqs_inf(~,x,par)
%Variable names
IRAK1  = x(1);
IRAK1p = x(2);
IKK    = x(3);
IKKa   = x(4);
NFkB   = x(5);
mIkBa  = x(6);
IkBa   = x(7);
mIL8   = x(8);
IL8    = x(9); 
%Parameter names
kirak1syn     = par(1);
kyrak1deg     = par(2);
kirak1ph1     = par(3);
kirak1ph2     = par(4);
kirak1pdeg    = par(5);
kikksyn       = par(6);
kikkdeg       = par(7);
kikkact       = par(8);
kikkadeg      = par(9);
knfkbgain     = par(10);
knfkbloss     = par(11);
kmikbatransc  = par(12);
kmikbadeg     = par(13);
kikbatransl   = par(14);
kikbaloss     = par(15);
kmil8transc1  = par(16);
kmil8transc2  = par(17);
kmil8deg      = par(18);
kil8transl    = par(19);
kil8deg       = par(20);
K1            = par(21);
K2            = par(22);
K3            = par(23);
Ntot          = par(24);
Flagellin     = 7.97;    
IL1B          = 1;    
%Differential equations
dx     = zeros(length(x),1);
dx(1)  = kirak1syn - kyrak1deg*IRAK1 - kirak1ph1*IRAK1*Flagellin - kirak1ph2*IRAK1*IL1B;
dx(2)  = kirak1ph1*IRAK1*Flagellin + kirak1ph2*IRAK1*IL1B - kirak1pdeg*IRAK1p;
dx(3)  = kikksyn - kikkact*IKK*IRAK1p - kikkdeg*IKK;
dx(4)  = kikkact*IKK*IRAK1p - kikkadeg*IKKa;
dx(5)  = knfkbgain*IKKa*((Ntot - NFkB)/(K1 + IkBa + IKKa)) - knfkbloss*((IkBa*NFkB)/(K2+NFkB));
dx(6)  = kmikbatransc*NFkB - kmikbadeg*mIkBa;
dx(7)  = kikbatransl*mIkBa - kikbaloss*IKKa*IkBa*((Ntot - NFkB)/(K3 + IkBa + IKKa));
dx(8)  = kmil8transc1 + kmil8transc2*NFkB - kmil8deg*mIL8;
dx(9)  = kil8transl*mIL8 - kil8deg*IL8;
end
